Main:
  // first initialize the RROS
  #include "RROScommands.bas"
  gosub InitCommands
  call InitSensors(0,0,0,0) // see last month
  gosub InitRB9robot // initialize the robot
  Dest = 40  // desired destination
  call Orientation(T1,T2,T3,Heading)
  call AngleDifference(Heading,Dest,dif)
  // turn to the desired direction
  rTurn dif // modify as discussed in the text
  // and move forward correcting as necessary 
  DirToMove = 0
  dif = 0
  while rRange()>24 // stop if object detected
    call Orientation(T1,T2,T3,Heading)
    call AngleDifference(Heading,DirToMove,dif)
    // note: this section is slightly improved over the article
    // it allows you to adjust the turn style for each wheel
    // which is important if the wheels are not well matched.
    if abs(dif)<5 // move forward if on course
      rForward 1
    elseif dif>0
      rCommand(SetTurnStyle,50)
      rTurn 1
    elseif dif<0
      rCommand(SetTurnStyle,50)
      rTurn -1
    endif
  wend
end

sub InitSensors(a,b,c,d)
  // The Utility MUST reside in the same directory as
  // the application unless you set up paths 
  print "Initializing..."
  SetCBtext("stop") // stop utility if running
  filerename ("RBsTurn.txt","SensorsTurn.txt")
  delay 100
  Spawn ("RBsensorUtility.exe","",P_NOWAIT) 
  // provide time for Utility to spawn
  delay 1000 
  ClearScr   
  //  The above code can be eliminated if you always
  //  start the Utility manually. We have code in
  //  the utility that SHOULD prevent a second copy
  //  from running - but for some unknown reason it
  //  is not 100% reliable.  The above code seems to
  //  always work on our machine.
  dim Sensors[28] //max available from utility
  dim Offsets[4]                 
  for i=0 to 27
    Sensors[i]=""
  next
  // allow setting certain angles to zero
  Offsets[0]=a
  Offsets[1]=b
  Offsets[2]=c
  Offsets[3]=d 
  // note: new initialization will be added here for voice commands 
return  

sub Orientation(&t1,&t2,&t3,&h)
   // Request readings
   SetCBtext("tilt")
   filerename ("RBsTurn.txt","SensorsTurn.txt")
   // and wait for it to respond
   while !fileexists("RBsTurn.txt") 
   wend
   delay 10
   // get the data
   s=getcbtext()
   // and parse it
   mfromstring Sensors,s,"," 
   t1 = round(Sensors[1])+Offsets[0]
   t2 = round(Sensors[2])+Offsets[1]
   t3 = round(Sensors[3])+Offsets[2]
   h  = round(Sensors[5])+Offsets[3]
   // ensure heading within normal range
   if h>359 then h-=360
   if h<0 then h+=360
return


InitRB9robot:
  rCommport 3
  delay 100
  SetTimeOut 50000
  rlocate 0,0 // 255,0
  rCommand(MotorSetup,SMALLDC+ENCODERS)  //+HMC6352 if you have a compass
  // rCommand(SensorSetup,0+PING) //3+FIVERANGE) //FIVERANGE+HMC6352) //0%10001000) rover 5
  rCommand (SensorSetup,SR04+FIVERANGE) // RB-9
  rCommand(SetMotorRamp,30)
  rCommand(SetClicksPerDiam,160)
  rCommand(SetClicksPer90,110) 
  rCommand(SetReducForwRight,0)
  rCommand(SetReducForwLeft,15)
  rCommand(SetReducBackRight,0)
  rCommand(SetReducBackLeft,0)
  rCommand(SetRotationTime,18) // 19)
  rCommand(SetMoveTime,12) //13) 
  rCommand(SetSpeed,60) //65) //,35)
  rCommand(SetSlowDownSpeed,50) //45) 
  rCommand(SetSlowDown2,40) 
  rCommand(SetCCdivisor,3)
  rCommand(SetBumpDist,10)
  rCommand(SetProxDist,20)
  rCommand(EnableCounters,1)
  rCommand(SetRobotAngle,90/2)
  rCommand(SetTurnStyle,100)
  rCommand(SetRROStimeout,200)  //15
return

sub AngleDifference(a,b,&d)
  // a=current, b=destination
  // d=difference
  if a=b
    d=0
    return
  elseif abs(a-b)>180
     if a>b
       b+=360
     else
       a+=360
     endif 
  endif
  d=b-a
return


