#include "RROScommands.bas"
gosub InitCommands

Main:
  call InitSensors(0,0,0,0) // set offsets
  // set Real=True to control real robot
  // set Real=False to control a simple simulation
  Real = false
  // proper initialization of a REAL robot will allow
  // this routine to control it by tilting the tablet
  print "Tilt Tablet Forward/Backward and Left/Right to control movement."
  if Real 
    gosub InitRB9robot 
  else
    rLocate 400,500
  endif
  gosub InitConstants
  while true    
    //call DetermineDesiredMovement(Speed,TurnAmount,Direction)
    call Orientation(Tilt1, Tilt2, Tilt3, Heading)
    // the following calculations assumes the tablet
    // is in the flat landscape mode
    if within(Tilt1,-10,10) then Tilt1=0
    if within(Tilt2,-10,10) then Tilt2=0
    xyString 20,100, Tilt1;Tilt2;" "
    Speed=abs(Tilt1)*3
    if Speed>99 then Speed=99
    TurnAmount=abs(Tilt2)*3
    if TurnAmount>99 then TurnAmount=99
    TurnAmount=100-TurnAmount
    Direction=255 // default (stop) condition
    // adjust these parameters if your tablet
    // is used in a different orientation
    if Tilt1<0 and Tilt2>0 then Direction=ForwRight
    if Tilt1<0 and Tilt2<0 then Direction=ForwLeft
    if Tilt1>0 and Tilt2>0 then Direction=RevRight
    if Tilt1>0 and Tilt2<0 then Direction=RevLeft
    if Tilt1<0 and Tilt2=0 then Direction=Forw
    if Tilt1>0 and Tilt2=0 then Direction=Rev
    if Tilt1=0 and Tilt2>0 then Direction=RotR
    if Tilt1=0 and Tilt2<0 then Direction=RotL
    //xyString 20,100, Tilt1;Tilt2;" "
    //xyString 20,100,Speed;TurnAmount;Direction;"  "
    // set robot's speed   
    rCommand(SetSpeed,Speed)
    // setup movement    
    if !Real
      if Direction=Forw
         rForward 1
      elseif Direction=Rev
         rForward -1
      elseif Direction=RotR
         rTurn 1
      elseif Direction=RotL
         rTurn -1
      elseif Direction=ForwRight
         rTurn 1
         rForward 1           
      elseif Direction=ForwLeft
         rTurn -1
         rForward 1          
      elseif Direction=RevRight
         rTurn -1
         rForward -1           
      elseif Direction=RevLeft
          rTurn 1
          rForward -1            
      elseif Direction=Stop
         rForward 0
      endif
    else
      if Direction=Forw
         rForward 1
      elseif Direction=Rev
         rForward -1
      elseif Direction=RotR
         rTurn 1
      elseif Direction=RotL
         rTurn -1
      elseif Direction=ForwRight
         rCommand(Drive,Direction+TurnAmount/2)
      elseif Direction=ForwLeft
         rCommand(Drive,Direction+TurnAmount/2)
      elseif Direction=RevRight
         rCommand(Drive,Direction+TurnAmount/2)
      elseif Direction=RevLeft
         rCommand(Drive,Direction+TurnAmount/2) 
      elseif Direction=Stop
         rForward 0
      endif
    endif     
  wend
end

sub InitSensors(a,b,c,d)
  // The Utility MUST reside in the same directory as
  // the application unless you set up paths 
  print "Initializing..."
  SetCBtext("stop") // stop utility if running
  filerename ("RBsTurn.txt","SensorsTurn.txt")
  delay 100
  Spawn ("RBsensorUtility.exe","",P_NOWAIT) 
  // provide time for Utility to spawn
  delay 1000 
  ClearScr   
  //  The above code can be eliminated if you always
  //  start the Utility manually. We have code in
  //  the utility that SHOULD prevent a second copy
  //  from running - but for some unknown reason it
  //  is not 100% reliable.  The above code seems to
  //  always work on our machine.
  dim Sensors[28] //max available from utility
  dim Offsets[4]                 
  for i=0 to 27
    Sensors[i]=""
  next
  // allow setting certain angles to zero
  Offsets[0]=a
  Offsets[1]=b
  Offsets[2]=c
  Offsets[3]=d 
  // note: new initialization will be added here for voice commands 
return  

sub Orientation(&t1,&t2,&t3,&h)
   // Request readings
   SetCBtext("tilt")
   filerename ("RBsTurn.txt","SensorsTurn.txt")
   // and wait for it to respond
   while !fileexists("RBsTurn.txt") 
   wend
   delay 10
   // get the data
   s=getcbtext()
   // and parse it
   mfromstring Sensors,s,"," 
   t1 = round(Sensors[1])+Offsets[0]
   t2 = round(Sensors[2])+Offsets[1]
   t3 = round(Sensors[3])+Offsets[2]
   h  = round(Sensors[5])+Offsets[3]
   // ensure heading within normal range
   if h>359 then h-=360
   if h<0 then h+=360
return


InitRB9robot:
  rCommport 3
  delay 100
  SetTimeOut 50000
  rlocate 0,0 // 255,0
  rCommand(MotorSetup,SMALLDC+ENCODERS)  //+HMC6352 if you have a compass
  // rCommand(SensorSetup,0+PING) //3+FIVERANGE) //FIVERANGE+HMC6352) //0%10001000) rover 5
  rCommand (SensorSetup,SR04+FIVERANGE) // RB-9
  rCommand(SetMotorRamp,30)
  rCommand(SetClicksPerDiam,160)
  rCommand(SetClicksPer90,110) 
  rCommand(SetReducForwRight,0)
  rCommand(SetReducForwLeft,15)
  rCommand(SetReducBackRight,0)
  rCommand(SetReducBackLeft,0)
  rCommand(SetRotationTime,18) // 19)
  rCommand(SetMoveTime,12) //13) 
  rCommand(SetSpeed,60) //65) //,35)
  rCommand(SetSlowDownSpeed,50) //45) 
  rCommand(SetSlowDown2,40) 
  rCommand(SetCCdivisor,3)
  rCommand(SetBumpDist,10)
  rCommand(SetProxDist,20)
  rCommand(EnableCounters,1)
  rCommand(SetRobotAngle,90/2)
  rCommand(SetTurnStyle,100)
  rCommand(SetRROStimeout,200)  //15
return

sub DetermineDesiredMovement(&s,&ta,&d)
   call ReadSensors(Tilt1, Tilt2, Tilt3, Heading)
   // the following calculations assumes the tablet
   // is in the flat landscape mode
   if within(Tilt1,-10,10) then Tilt1=0
   if within(Tilt2,-10,10) then Tilt2=0
   s=abs(Tilt1)*3
   if s>99 then s=99
   ta=abs(Tilt2)*3
   if ta>99 then ta=99
   d=255 // default (stop) condition
   // adjust these parameters if your tablet
   // is used in a different orientation
   if Tilt1<0 and Tilt2>0 then d=_ForwRight
   if Tilt1<0 and Tilt2<0 then d=_ForwLeft
   if Tilt1>0 and Tilt2>0 then d=_RevRight
   if Tilt1>0 and Tilt2<0 then d=_RevLeft
   if Tilt1<0 and Tilt2=0 then d=_Forw
   if Tilt1>0 and Tilt2=0 then d=_Rev
   if Tilt1=0 and Tilt2>0 then d=_RotR
   if Tilt1=0 and Tilt2<0 then d=_RotL
return

InitConstants:
  ForwRight = 0<<6
  ForwLeft = 1<<6
  RevRight = 2<<6
  RevLeft = 3<<6
  RotR = 1
  RotL = 2
  Forw = 3
  Rev = 4 
  Stop = 255
  Drive = 159 
return

